@@ -1,10 +1,12 @@
+from __future__ import unicode_literals
+import pickle
 
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.core.management import call_command
-from django.db import models, router
+from django.db import connections, models, router
 from django.db.utils import IntegrityError
-from django.utils.translation import ugettext
+from django.utils.translation import ugettext_lazy as _
 
 from mutant.contrib.text.models import CharFieldDefinition
 from mutant.contrib.related.models import ForeignKeyDefinition
@@ -13,6 +15,7 @@ from mutant.models.model import (ModelDefinition, OrderingFieldDefinition,
     UniqueTogetherDefinition, BaseDefinition)
 from mutant.tests.models.utils import (BaseModelDefinitionTestCase,
     skipUnlessMutantModelDBFeature)
+from mutant.test.utils import CaptureQueriesContext
 
 
 try:
@@ -33,74 +36,197 @@ except ImportError:
         finally:
             sys.stderr = stderr
 
-class ModelDefinitionManipulationTest(BaseModelDefinitionTestCase):
-    
+
+class Mixin(object):
+    def method(self):
+        return 'Mixin'
+
+
+class ConcreteModel(models.Model):
+    concrete_model_field = models.NullBooleanField()
+
+    class Meta:
+        app_label = 'mutant'
+
+
+class ProxyModel(ConcreteModel):
+    class Meta:
+        proxy = True
+
+
+class AbstractModel(models.Model):
+    abstract_model_field = models.CharField(max_length=5)
+
+    class Meta:
+        abstract = True
+
+    def method(self):
+        return 'AbstractModel'
+
+
+class AbstractConcreteModelSubclass(ConcreteModel):
+    abstract_concrete_model_subclass_field = models.CharField(max_length=5)
+
+    class Meta:
+        abstract = True
+
+
+class ModelSubclassWithTextField(models.Model):
+    abstract_model_field = models.TextField()
+    second_field = models.NullBooleanField()
+
+    class Meta:
+        abstract = True
+
+
+class ModelDefinitionTest(BaseModelDefinitionTestCase):
     def test_model_class_creation_cache(self):
         Model = self.model_def.model_class()
         self.assertEqual(Model, self.model_def.model_class())
-        
         self.assertNotEqual(Model, self.model_def.model_class(force_create=True))
 
+    def get_model_db_table(self):
+        return self.model_def.model_class()._meta.db_table
+
     def test_rename_model(self):
         """
         Make sure changing the app_label or object_name renames the associated
         table
         """
-        def get_table_name():
-            return self.model_def.model_class()._meta.db_table
-        
         db = router.db_for_read(self.model_def.model_class())
-        
-        table_name = get_table_name()
+        table_name = self.get_model_db_table()
         self.model_def.app_label = 'myapp'
         self.model_def.save()
         self.assertTableDoesntExists(db, table_name)
-        table_name = get_table_name()
+        table_name = self.get_model_db_table()
         self.assertTableExists(db, table_name)
-        
+
         self.model_def.object_name = 'MyModel'
         self.model_def.save()
         self.assertTableDoesntExists(db, table_name)
-        table_name = get_table_name()
+        table_name = self.get_model_db_table()
         self.assertTableExists(db, table_name)
-        
+
         self.model_def.delete()
         self.assertTableDoesntExists(db, table_name)
-        
+
+    def test_db_table(self):
+        """
+        Asserts that the db_table field is correctly handled
+        """
+        db = router.db_for_read(self.model_def.model_class())
+        generated_table_name = self.get_model_db_table()
+        self.model_def.db_table = 'test_db_table'
+        self.model_def.save()
+        self.assertTableDoesntExists(db, generated_table_name)
+        self.assertTableExists(db, 'test_db_table')
+        self.model_def.db_table = None
+        self.model_def.save()
+        self.assertTableDoesntExists(db, 'test_db_table')
+        self.assertTableExists(db, generated_table_name)
+
     def test_fixture_loading(self):
         call_command('loaddata', 'fixture_loading_test', verbosity=0, commit=False)
         model_def = ModelDefinition.objects.get(app_label='myfixtureapp',
                                                 object_name='MyFixtureModel')
         Model = model_def.model_class()
         Model.objects.create()
-        
+
     def test_verbose_name(self):
         Model = self.model_def.model_class()
-        
+
         self.assertEqual(Model._meta.verbose_name, self.model_def.model)
         self.assertEqual(Model._meta.verbose_name_plural, "%ss" % self.model_def.model)
-        
-        self.model_def.verbose_name = u'MyMoDeL'
-        self.model_def.verbose_name_plural = u'MyMoDeLZ0Rs'
+
+        self.model_def.verbose_name = 'MyMoDeL'
+        self.model_def.verbose_name_plural = 'MyMoDeLZ0Rs'
         self.model_def.save()
-        
-        self.assertEqual(Model._meta.verbose_name, ugettext(u'MyMoDeL'))
-        self.assertEqual(Model._meta.verbose_name_plural, ugettext(u'MyMoDeLZ0Rs'))
-        
+
+        self.assertEqual(Model._meta.verbose_name, _('MyMoDeL'))
+        self.assertEqual(Model._meta.verbose_name_plural, _('MyMoDeLZ0Rs'))
+
     def test_multiple_model_definition(self):
         """
         Make sure multiple model definition can coexists
         """
         other_model_def = ModelDefinition.objects.create(app_label='app',
                                                          object_name='OtherModel')
-        
         self.assertNotEqual(other_model_def.model_class(),
                             self.model_def.model_class())
-        
         self.assertNotEqual(other_model_def.model_ct, self.model_def.model_ct)
-            
+
+    def test_natural_key(self):
+        natural_key = self.model_def.natural_key()
+        self.assertEqual(ModelDefinition.objects.get_by_natural_key(*natural_key),
+                         self.model_def)
+
+    def test_deletion(self):
+        # Add a an extra field to make sure no alter statements are issued
+        CharFieldDefinition.objects.create(
+            model_def=self.model_def,
+            name='field',
+            max_length=10
+        )
+        # Add a base with a field to make sure no alter statements are issued
+        BaseDefinition.objects.create(
+            model_def=self.model_def,
+            base=AbstractModel
+        )
+        model_cls = self.model_def.model_class()
+        self.assertModelTablesExist(model_cls)
+        db = router.db_for_write(model_cls)
+        table_name = self.get_model_db_table()
+        connection = connections[db]
+        with CaptureQueriesContext(connection) as captured_queries:
+            self.model_def.delete()
+        # ensure that no ALTER queries where issues during deletion of model_def,
+        # that is, check that the table was not deleted on column at a time before
+        # the entire table was dropped.
+        self.assertFalse(any('ALTER' in query['sql'] for query in captured_queries))
+        self.assertTableDoesntExists(db, table_name)
+
+
+class ModelDefinitionManagerTest(BaseModelDefinitionTestCase):
+    def test_fields_creation(self):
+        char_field = CharFieldDefinition(name='name', max_length=10)
+        ct_ct = ContentType.objects.get_for_model(ContentType)
+        fk_field = ForeignKeyDefinition(name='ct', to=ct_ct)
+        model_def = ModelDefinition.objects.create(fields=(char_field, fk_field),
+                                                   app_label='app',
+                                                   object_name='OtherModel')
+        model_cls = model_def.model_class()
+        db = router.db_for_write(model_cls)
+        table = model_cls._meta.db_table
+        column = model_cls._meta.get_field('name').get_attname_column()[1]
+        # Make sure column was created
+        self.assertColumnExists(db, table, column)
+        # Make sure field definitions were created
+        self.assertIsNotNone(char_field.pk)
+        self.assertIsNotNone(fk_field.pk)
+
+    def test_bases_creation(self):
+        mixin_base = BaseDefinition(base=Mixin)
+        abstract_base = BaseDefinition(base=AbstractModel)
+        abstract_concrete_base = BaseDefinition(base=AbstractConcreteModelSubclass)
+        model_def = ModelDefinition.objects.create(
+            bases=(mixin_base, abstract_base, abstract_concrete_base),
+            app_label='app',
+            object_name='OtherModel'
+        )
+        model = model_def.model_class()
+        self.assertModelTablesColumnExists(model, 'abstract_model_field')
+        self.assertModelTablesColumnDoesntExists(model, 'concrete_model_field')
+        self.assertModelTablesColumnExists(model, 'abstract_concrete_model_subclass_field')
+
+    def test_primary_key_override(self):
+        field = CharFieldDefinition(name='name', max_length=32, primary_key=True)
+        model_def = ModelDefinition.objects.create(fields=(field,),
+                                                   app_label='app',
+                                                   object_name='OtherModel')
+        self.assertEqual(model_def.model_class()._meta.pk.name, 'name')
+
+
 class ModelValidationTest(BaseModelDefinitionTestCase):
-    
     def test_installed_app_override_failure(self):
         """
         Make sure we can't save a model definition with an app_label of
@@ -109,38 +235,41 @@ class ModelValidationTest(BaseModelDefinitionTestCase):
         self.model_def.app_label = 'mutant'
         self.assertRaises(ValidationError, self.model_def.clean)
 
-class ModelClassProxyProxyTests(BaseModelDefinitionTestCase):
-    
+
+class ModelClassProxyTest(BaseModelDefinitionTestCase):
+    def test_pickling(self):
+        """
+        Make sure _ModelClassProxy can be pickled correctly. This is required
+        to allow a model definition to subclass a MutableModel.
+        """
+        Model = self.model_def.model_class()
+        pickled = pickle.dumps(Model)
+        self.assertEqual(pickle.loads(pickled), Model)
+
     def test_proxy_interactions(self):
         CharFieldDefinition.objects.create(model_def=self.model_def,
                                            name="name", max_length=10)
         Model = self.model_def.model_class()
-        
         sergei = Model.objects.create(name='Sergei')
-        
         halak = Model(name='Halak')
         halak.save()
-        
-        assert issubclass(Model, models.Model)
-        
-        assert issubclass(Model, MutableModel)
-        
-        assert unicode(Model) == u"<class 'mutant.apps.app.models.Model'>"
-        
-        assert sergei == Model.objects.get(name='Sergei')
-        
+        self.assertTrue(issubclass(Model, models.Model))
+        self.assertTrue(issubclass(Model, MutableModel))
+        self.assertEqual("<class 'mutant.apps.app.models.Model'>",
+                         unicode(Model))
+        self.assertEqual(sergei, Model.objects.get(name='Sergei'))
+
         class A(object):
-            
             class_model = Model
-            
+
             def __init__(self, model):
                 self.model = model
-        
+
         a = A(Model)
-        
-        assert Model == a.model
-        assert Model == A.class_model
-        
+
+        self.assertEqual(Model, a.model)
+        self.assertEqual(Model, A.class_model)
+
         a.model = Model # Assign a proxy
         a.model = a.model # Assign a Model
         a.model = 4
@@ -148,33 +277,31 @@ class ModelClassProxyProxyTests(BaseModelDefinitionTestCase):
     def test_definition_deletion(self):
         CharFieldDefinition.objects.create(model_def=self.model_def,
                                            name="name", max_length=10)
-        
+
         Model = self.model_def.model_class()
         db = router.db_for_write(Model)
         instance = Model.objects.create(name="Quebec")
         table_name = Model._meta.db_table
         self.model_def.delete()
         self.assertTableDoesntExists(db, table_name)
-        
+
         with self.assertRaises(AttributeError):
             Model(name="name")
-        
+
         with self.assertRaises(AttributeError):
             Model.objects.all()
-           
+
         with self.assertRaises(ValidationError):
             instance.clean()
-            
+
         with self.assertRaises(ValidationError):
             instance.save()
-        
+
         with self.assertRaises(ValidationError):
             instance.delete()
-            
-        self.setUp() # Recreate the model definition to delete it in tearDown
+
 
 class OrderingDefinitionTest(BaseModelDefinitionTestCase):
-    
     def setUp(self):
         super(OrderingDefinitionTest, self).setUp()
         self.f1 = CharFieldDefinition.objects.create(model_def=self.model_def,
@@ -186,20 +313,20 @@ class OrderingDefinitionTest(BaseModelDefinitionTestCase):
 
     def test_clean(self):
         ordering = OrderingFieldDefinition(model_def=self.model_def)
-        
+        # Random
         ordering.lookup = '?'
         ordering.clean()
-        
+        # By f1
         ordering.lookup = 'f1'
         ordering.clean()
-        
+        # By f2 app label
         ordering.lookup = 'f2__app_label'
         ordering.clean()
-        
+        # Inexistent field
         with self.assertRaises(ValidationError):
-            ordering.lookup = 'f3'
+            ordering.lookup = 'inexistent_field'
             ordering.clean()
-            
+        # Inexistent field of an existent field
         with self.assertRaises(ValidationError):
             ordering.lookup = 'f2__higgs_boson'
             ordering.clean()
@@ -211,43 +338,42 @@ class OrderingDefinitionTest(BaseModelDefinitionTestCase):
         ct_ct = ContentType.objects.get_for_model(ContentType) #contenttypes
         Model.objects.create(f1='Simon', f2=ct_ct)
         Model.objects.create(f1='Alexander', f2=model_ct)
-        
-        
         # Instances should be sorted by id
-        self.assertQuerysetEqual(Model.objects.values('f1'),
-                                 [u'Simon', u'Alexander'],
-                                 transform=lambda x: x['f1'], ordered=True)
-        
+        self.assertSequenceEqual(
+            Model.objects.values_list('f1', flat=True),
+            ('Simon', 'Alexander')
+        )
         # Instances should be sorted by f1 and not id
         f1_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                              lookup='f1')
-        self.assertQuerysetEqual(Model.objects.values('f1'),
-                                 [u'Alexander', u'Simon'],
-                                 transform=lambda x: x['f1'], ordered=True)
-        
+        self.assertSequenceEqual(
+            Model.objects.values_list('f1', flat=True),
+            ('Alexander', 'Simon')
+        )
         # Swap the ordering to descending
         f1_ordering.descending = True
         f1_ordering.save()
-        self.assertQuerysetEqual(Model.objects.values('f1'),
-                                 [u'Simon', u'Alexander'],
-                                 transform=lambda x: x['f1'], ordered=True)
+        self.assertSequenceEqual(
+            Model.objects.values_list('f1', flat=True),
+            ('Simon', 'Alexander')
+        )
         f1_ordering.delete()
-        
         # Order by f2__app_label
         f2_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                              lookup='f2__app_label')
-        self.assertQuerysetEqual(Model.objects.values('f1'),
-                                 [u'Alexander', u'Simon'],
-                                 transform=lambda x: x['f1'], ordered=True)
-        
+        self.assertSequenceEqual(
+            Model.objects.values_list('f1', flat=True),
+            ('Alexander', 'Simon')
+        )
         # Swap the ordering to descending
         f2_ordering.descending = True
         f2_ordering.save()
-        self.assertQuerysetEqual(Model.objects.values('f1'),
-                                 [u'Simon', u'Alexander'],
-                                 transform=lambda x: x['f1'], ordered=True)
+        self.assertSequenceEqual(
+            Model.objects.values_list('f1', flat=True),
+            ('Simon', 'Alexander')
+        )
         f2_ordering.delete()
-    
+
     @skipUnlessMutantModelDBFeature('supports_joins')
     def test_multiple_ordering(self):
         Model = self.model_def.model_class()
@@ -257,52 +383,58 @@ class OrderingDefinitionTest(BaseModelDefinitionTestCase):
         Model.objects.create(f1='Alexander', f2=model_ct)
         Model.objects.create(f1='Julia', f2=ct_ct)
         Model.objects.create(f1='Alexander', f2=ct_ct)
-        
+        # Orderings
         f1_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                              lookup='f1')
         f2_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                              lookup='f2__app_label')
-        
-        
-        self.assertQuerysetEqual(Model.objects.values('f1', 'f2__app_label'),
-                                 [(u'Alexander', u'app'), (u'Alexander', u'contenttypes'),
-                                  (u'Julia', u'contenttypes'), (u'Simon', u'contenttypes')],
-                                 transform=lambda x: (x['f1'], x['f2__app_label']),
-                                 ordered=True)
-        
+        self.assertSequenceEqual(
+            Model.objects.values_list('f1', 'f2__app_label'), (
+                ('Alexander', 'app'),
+                ('Alexander', 'contenttypes'),
+                ('Julia', 'contenttypes'),
+                ('Simon', 'contenttypes')
+            )
+        )
         # Swap the ordering to descending
         f2_ordering.descending = True
         f2_ordering.save()
-        self.assertQuerysetEqual(Model.objects.values('f1', 'f2__app_label'),
-                                 [(u'Alexander', u'contenttypes'), (u'Alexander', u'app'),
-                                  (u'Julia', u'contenttypes'), (u'Simon', u'contenttypes')],
-                                 transform=lambda x: (x['f1'], x['f2__app_label']),
-                                 ordered=True)
-        
+        self.assertSequenceEqual(
+            Model.objects.values_list('f1', 'f2__app_label'), (
+                ('Alexander', 'contenttypes'),
+                ('Alexander', 'app'),
+                ('Julia', 'contenttypes'),
+                ('Simon', 'contenttypes')
+            )
+        )
         # Swap order
         f1_ordering.order, f2_ordering.order = f2_ordering.order, f1_ordering.order
         f1_ordering.save()
         f2_ordering.save()
-        self.assertQuerysetEqual(Model.objects.values('f1', 'f2__app_label'),
-                                 [(u'Alexander', u'contenttypes'), (u'Julia', u'contenttypes'),
-                                  (u'Simon', u'contenttypes'), (u'Alexander', u'app')],
-                                 transform=lambda x: (x['f1'], x['f2__app_label']),
-                                 ordered=True)
-        
+        self.assertSequenceEqual(
+            Model.objects.values_list('f1', 'f2__app_label'), (
+                ('Alexander', 'contenttypes'),
+                ('Julia', 'contenttypes'),
+                ('Simon', 'contenttypes'),
+                ('Alexander', 'app')
+            )
+        )
         # Swap the ordering to descending
         f1_ordering.descending = True
         f1_ordering.save()
-        self.assertQuerysetEqual(Model.objects.values('f1', 'f2__app_label'),
-                                 [(u'Simon', u'contenttypes'), (u'Julia', u'contenttypes'),
-                                  (u'Alexander', u'contenttypes'), (u'Alexander', u'app')],
-                                 transform=lambda x: (x['f1'], x['f2__app_label']),
-                                 ordered=True)
-        
+        self.assertSequenceEqual(
+            Model.objects.values_list('f1', 'f2__app_label'), (
+                ('Simon', 'contenttypes'),
+                ('Julia', 'contenttypes'),
+                ('Alexander', 'contenttypes'),
+                ('Alexander', 'app')
+            )
+        )
         f1_ordering.delete()
         f2_ordering.delete()
 
+
 class UniqueTogetherDefinitionTest(BaseModelDefinitionTestCase):
-    
     def setUp(self):
         super(UniqueTogetherDefinitionTest, self).setUp()
         self.f1 = CharFieldDefinition.objects.create(model_def=self.model_def,
@@ -319,14 +451,25 @@ class UniqueTogetherDefinitionTest(BaseModelDefinitionTestCase):
         """
         other_model_def = ModelDefinition.objects.create(app_label='app',
                                                          object_name='OtherModel')
-        
         f2 = CharFieldDefinition.objects.create(model_def=other_model_def,
                                                 name='f2', max_length=25)
-        self.ut.field_defs.add(self.f1, f2)
+        self.ut.field_defs = (self.f1, f2)
         with self.assertRaises(ValidationError):
             self.ut.clean()
         other_model_def.delete()
 
+    def test_db_column(self):
+        """
+        Make sure a unique index creation works correctly when using a custom
+        `db_column`. This is needed for unique FK's columns.
+        """
+        self.f2.db_column = 'f2_column'
+        self.f2.save()
+        self.ut.field_defs = (self.f1, self.f2)
+        self.f2.db_column = 'f2'
+        self.f2.save()
+        self.ut.delete()
+
     def test_cannot_create_unique(self):
         """
         Creating a unique key on a table with duplicate 
@@ -336,24 +479,24 @@ class UniqueTogetherDefinitionTest(BaseModelDefinitionTestCase):
         self.Model.objects.create(f1='a', f2='b')
         with captured_stderr():
             with self.assertRaises(IntegrityError):
-                self.ut.field_defs.add(self.f1, self.f2)
-    
+                self.ut.field_defs = (self.f1, self.f2)
+
     def test_cannot_insert_duplicate_row(self):
         """
         Inserting a duplicate rows shouldn't work
         """
         self.Model.objects.create(f1='a', f2='b')
-        self.ut.field_defs.add(self.f1, self.f2)
+        self.ut.field_defs = (self.f1, self.f2)
         with captured_stderr():
             with self.assertRaises(IntegrityError):
                 self.Model.objects.create(f1='a', f2='b')
-    
+
     def test_cannot_remove_unique(self):
         """
         Removing a unique constraint that cause 
         duplicate rows shouldn't work
         """
-        self.ut.field_defs.add(self.f1, self.f2)
+        self.ut.field_defs = (self.f1, self.f2)
         self.Model.objects.create(f1='a', f2='b')
         self.Model.objects.create(f1='a', f2='c')
         with captured_stderr():
@@ -366,119 +509,111 @@ class UniqueTogetherDefinitionTest(BaseModelDefinitionTestCase):
         validation
         """
         self.Model.objects.create(f1='a', f2='b')
-        self.ut.field_defs.add(self.f1, self.f2)
+        self.ut.field_defs = self.f1, self.f2
         self.ut.field_defs.clear()
         self.Model.objects.create(f1='a', f2='b')
-    
-class Mixin(object):
-    
-    def method(self):
-        return 'Mixin'
 
-class ModelProxy(CharFieldDefinition):
-    
-    class Meta:
-        proxy = True
 
-class ModelSubclass(models.Model):
-    
-    field = models.CharField(max_length=5)
-    
-    class Meta:
-        abstract = True
-    
-    def method(self):
-        return 'ModelSubclass'
-    
-class ModelSubclassWithTextField(models.Model):
-    
-    field = models.TextField()
-    second_field = models.NullBooleanField()
-    
-    class Meta:
-        abstract = True
-
-class MutableModelSubclass(MutableModel):
-    pass
-        
 class BaseDefinitionTest(BaseModelDefinitionTestCase):
-    
     def test_clean(self):
-        bd = BaseDefinition()
-        
+        """
+        Ensure `BaseDefinition.clean` works correctly.
+        """
+        bd = BaseDefinition(model_def=self.model_def)
         # Base must be a class
         bd.base = BaseDefinitionTest.test_clean
-        self.assertRaises(ValidationError, bd.clean)
-        
-        # Subclasses of MutableModel are not valid bases
-        bd.base = MutableModelSubclass
-        self.assertRaises(ValidationError, bd.clean)
-        
+        self.assertRaisesMessage(ValidationError,
+                                 _('Base must be a class.'), bd.clean)
+        # Subclasses of MutableModel are valid bases
+        bd.base = ModelDefinition.objects.create(app_label='app',
+                                                 object_name='AnotherModel').model_class()
+        try:
+            bd.clean()
+        except ValidationError:
+            self.fail('MutableModel subclasses are valid bases.')
+        # But model definition can't be bases of themselves
+        bd.base = self.model_def.model_class()
+        self.assertRaisesMessage(ValidationError,
+                                 _("A model definition can't be a base of "
+                                   "itself."), bd.clean)
         # Mixin objets are valid bases
         bd.base = Mixin
-        bd.clean()
+        try:
+            bd.clean()
+        except ValidationError:
+            self.fail('Mixin objets are valid bases.')
+        # Abstract model subclasses are valid bases
+        bd.base = AbstractModel
+        try:
+            bd.clean()
+        except ValidationError:
+            self.fail('Abstract Model are valid bases')
+        # Proxy model are not valid bases
+        bd.base = ProxyModel
+        self.assertRaisesMessage(ValidationError,
+                                 _("Base can't be a proxy model."), bd.clean)
+
+    @skipUnlessMutantModelDBFeature('supports_joins')
+    def test_mutable_model_base(self):
+        another_model_def = ModelDefinition.objects.create(app_label='app',
+                                                           object_name='AnotherModel')
+        AnotherModel = another_model_def.model_class()
+        CharFieldDefinition.objects.create(model_def=self.model_def,
+                                           name='f1', max_length=25)
+        base_definition = BaseDefinition(model_def=another_model_def)
+        base_definition.base = self.model_def.model_class()
+        base_definition.save()
+        another_model = AnotherModel.objects.create(f1='Martinal')
+        self.assertTrue(AnotherModel.objects.exists())
+        CharFieldDefinition.objects.create(model_def=self.model_def,
+                                           name='f2', max_length=25, null=True)
+        another_model = AnotherModel.objects.get(pk=another_model.pk)
+        self.assertIsNone(another_model.f2)
+        another_model.f2 = 'Placebo'
+        another_model.save()
 
-        # Model subclasses are valid bases if they are abstract
-        bd.base = ModelSubclass
-        bd.clean()
-        
-        # Model subclasses that are not abstract are invalid
-        bd.base = CharFieldDefinition
-        self.assertRaises(ValidationError, bd.clean)
-        bd.base = ModelProxy
-        self.assertRaises(ValidationError, bd.clean)
-    
     def test_base_inheritance(self):
         Model = self.model_def.model_class()
-        
         BaseDefinition.objects.create(model_def=self.model_def,
                                       base=Mixin)
         self.assertTrue(issubclass(Model, Mixin))
-        
         BaseDefinition.objects.create(model_def=self.model_def,
-                                      base=ModelSubclass)
+                                      base=AbstractModel)
         self.assertTrue(issubclass(Model, Mixin) and
-                        issubclass(Model, ModelSubclass))
-    
+                        issubclass(Model, AbstractModel))
+
     def test_base_ordering(self):
         Model = self.model_def.model_class()
-        
         BaseDefinition.objects.create(model_def=self.model_def,
                                       base=Mixin, order=2)
         model_subclass_def = BaseDefinition.objects.create(model_def=self.model_def,
-                                                           base=ModelSubclass,
+                                                           base=AbstractModel,
                                                            order=1)
-        
         instance = Model()
-        self.assertEqual('ModelSubclass', instance.method())
-        
+        self.assertEqual('AbstractModel', instance.method())
         model_subclass_def.order = 3
         model_subclass_def.save()
         instance = Model()
         self.assertEqual('Mixin', instance.method())
-        
+
     def test_abstract_field_inherited(self):
         bd = BaseDefinition.objects.create(model_def=self.model_def,
-                                           base=ModelSubclass)
-        
+                                           base=AbstractModel)
         Model = self.model_def.model_class()
-        
-        Model.objects.create(field='value')
-        
+        Model.objects.create(abstract_model_field='value')
         # Test column alteration and addition by replacing the base with
         # a new one with a field with the same name and a second field.
         bd.base = ModelSubclassWithTextField
         bd.save()
-        Model.objects.get(field='value')
+        Model.objects.get(abstract_model_field='value')
         # The original CharField should be replaced by a TextField with no
         # max_length and a second field should be added
-        Model.objects.create(field='another one bites the dust',
+        Model.objects.create(abstract_model_field='another one bites the dust',
                              second_field=True)
-        
         # Test column deletion by deleting the base
         # This should cause the model to loose all it's fields and the table
         # to loose all it's columns
         bd.delete()
         self.assertEqual(list(Model.objects.values_list()),
                          [(instance.id,) for instance in Model.objects.all()])
-        self.assertColumnDoesntExists(Model._meta.db_table, 'field')
+        self.assertModelTablesColumnDoesntExists(Model, 'field')
